@@ -20,7 +20,7 @@
  */
 #ifndef lint
 static const char rcsid[] _U_ =
-    "@(#) $Header: /tcpdump/master/libpcap/pcap-snoop.c,v 1.55.2.3 2008-04-14 20:41:52 guy Exp $ (LBL)";
+    "@(#) $Header: /tcpdump/master/libpcap/pcap-snoop.c,v 1.59 2008-12-02 16:25:14 guy Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -226,8 +226,8 @@ pcap_activate_snoop(pcap_t *p)
 		    pcap_strerror(errno));
 		goto bad;
 	}
-	if (handle->opt.buffer_size != 0)
-		v = handle->opt.buffer_size;
+	if (p->opt.buffer_size != 0)
+		v = p->opt.buffer_size;
 	else
 		v = 64 * 1024;	/* default to 64K buffer size */
 	(void)setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (char *)&v, sizeof(v));
