@@ -37,6 +37,20 @@ static const char rcsid[] _U_ =
 #include "config.h"
 #endif
 
+#ifdef WIN32
+#include <pcap-stdinc.h>
+#else /* WIN32 */
+#if HAVE_INTTYPES_H
+#include <inttypes.h>
+#elif HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#ifdef HAVE_SYS_BITYPES_H
+#include <sys/bitypes.h>
+#endif
+#include <sys/types.h>
+#endif /* WIN32 */
+
 #include <errno.h>
 #include <memory.h>
 #include <stdio.h>
@@ -693,6 +707,21 @@ static const char rcsid[] _U_ =
  */
 #define LINKTYPE_AOS				222
 
+/*
+ * Wireless HART (Highway Addressable Remote Transducer)
+ * From the HART Communication Foundation
+ * IES/PAS 62591
+ *
+ * Requested by Sam Roberts <vieuxtech@gmail.com>.
+ */
+#define LINKTYPE_WIHART				223
+
+/*
+ * Fibre Channel FC-2 frames, beginning with a Frame_Header.
+ * Requested by Kahou Lei <kahou82@gmail.com>.
+ */
+#define LINKTYPE_FC_2				224
+
 
 static struct linktype_map {
 	int	dlt;
@@ -1020,6 +1049,12 @@ static struct linktype_map {
 	/* AOS Space Data Link Protocol */
 	{ DLT_AOS,		LINKTYPE_AOS },
 
+	/* Wireless HART */
+	{ DLT_WIHART,		LINKTYPE_WIHART },
+
+	/* Fibre Channel FC-2 frames */
+	{ DLT_FC_2,		LINKTYPE_FC_2 },
+
 	{ -1,			-1 }
 };
 
